﻿using UrlCleaner;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;

namespace UrlCleaner 
{
    /// <summary>
    /// AmaZonk status log viewer dialog.
    /// 
    /// Author: Dennis Lang - 2013
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// </summary>
    public partial class UrlDlg : Form
    {
        public UrlDlg()
        {
            InitializeComponent();
        }

        protected override void OnShown(EventArgs e)
        {
            clearBtn.Visible = (this.ClearLogEvent != null);
            base.OnShown(e);
        }

        const int sColUrl = 2;
        public List<string> UrlList
        {
            set 
            {
                this.urlListView.Items.Clear();
                string cntStr = "";
                foreach (string url in value)
                {
                    int cnt = this.urlListView.Items.Count;
                    cntStr = cnt.ToString("#,##0");
                    ListViewItem lvItem = this.urlListView.Items.Add(cntStr);
                    lvItem.SubItems.Add(string.Empty);
                    lvItem.SubItems.Add(url);
                    lvItem.UseItemStyleForSubItems = false;
                    lvItem.SubItems[sColUrl].BackColor = (((cnt / 5) & 1) == 1) ? Color.White : Color.FromArgb(0xe0ffe0);
                }
                this.lengthTx.Text = cntStr;
            }

            get
            {
                List<string> outList = new List<string>();
                foreach (ListViewItem lvItem in this.urlListView.Items)
                {
                    outList.Add(lvItem.SubItems[sColUrl].Text);
                }
                return outList;
            }
        }

        public delegate void CleaLog(object sender, EventArgs e);
        public event CleaLog ClearLogEvent;

        private void closeBtn_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void clearBtn_Click(object sender, EventArgs e)
        {
            if (this.ClearLogEvent != null)
                this.ClearLogEvent(sender, e);
        }

        private void urlListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.urlListView.SelectedItems.Count == 0)
                this.urlText.Text = string.Empty;
            else
                this.urlText.Text = this.urlListView.SelectedItems[0].SubItems[2].Text;
        }

        private void urlListView_DoubleClick(object sender, EventArgs e)
        {
            if (this.urlListView.SelectedItems.Count == 0)
                this.urlText.Text = string.Empty;
            else
            {
                this.urlText.Text = this.urlListView.SelectedItems[0].SubItems[2].Text;
                openViewer(this.urlText.Text);
            }
        }

        private void openViewer(string url)
        { 
            try
            {
                int httpIdx = url.IndexOf("http");
                Uri uri = new Uri(url.Substring(httpIdx));
                System.Diagnostics.Process.Start(uri.ToString());
            }
            catch { }
        }

        private void urlText_DoubleClick(object sender, EventArgs e)
        {
            openViewer(this.urlText.Text);
        }

        private void saveBtn_Click(object sender, EventArgs e)
        {
            List<string> listString = UrlList;


            if (listString.Count != 0 && saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                String filePath = saveFileDialog.FileName;
                String ext = Path.GetExtension(filePath).ToLower();
                String dateStr = DateTime.Now.ToString("s");

                try
                {
                    
                    Stream saveToStream = new FileStream(filePath, FileMode.Create, FileAccess.Write, FileShare.None);

                    switch (ext)
                    {
                        case ".xml":
                            var xml = new XmlSerializer(listString.GetType());
                            xml.Serialize(saveToStream, listString);
                            break;
                        case ".html":
                        case ".htm":
                            using (StreamWriter saveToText = new StreamWriter(saveToStream))
                            { 
                                saveToText.WriteLine(String.Format("<html><body><h1>AmaZonk Url List {0}</h1><p>", dateStr));
                                foreach (String str in listString)
                                    saveToText.WriteLine(String.Format("<li><a href='{0}'>{0}</a>", str));
                                saveToText.WriteLine("</body></html>");
                            }
                            openViewer(filePath);
                            break;
                        case ".bin":
                        case ".dat":
                            IFormatter formatter = new BinaryFormatter();
                            formatter.Serialize(saveToStream, listString);
                            break;
                        case ".txt":
                        default:
                            using (StreamWriter saveToText = new StreamWriter(saveToStream))
                            {
                                saveToText.WriteLine("AmaZonk Url List " + dateStr);
                                foreach (String str in listString)
                                    saveToText.WriteLine(str);
                            }
                            break;
                    }


                    saveToStream.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }

            }
        }
      
    }
}
