﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace UrlCleaner
{
    /// <summary>
    /// UI component to manage a list of filters.
    /// Has two columns, Include and Exclude patterns.
    /// 
    /// Author: Dennis Lang - 2011
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    public partial class FilterUI : UserControl
    {
        public FilterUI()
        {
            InitializeComponent();
        }

        [CategoryAttribute("General"),
        DescriptionAttribute("Title")]
        public string Title
        {
            set { this.title.Text = value; }
            get { return this.title.Text; }
        }

        [CategoryAttribute("General"),
        DescriptionAttribute("Value")]
        public class CheckedValue
        {
            public bool enable;
            public string value;
        }

        public CheckedValue[] List
        {
            set
            {
                this.listView.Items.Clear();
                foreach (CheckedValue ckValue in value)
                {
                    this.listView.Items.Add(ckValue.value).Checked = ckValue.enable;
                }
            }

            get
            {
                CheckedValue[] list = new CheckedValue[this.listView.Items.Count];
                for (int idx = 0; idx != this.listView.Items.Count; idx++)
                {
                    list[idx] = new CheckedValue() { enable = this.listView.Items[idx].Checked, value = this.listView.Items[idx].Text };
                }
                return list;
            }
        }


        [CategoryAttribute("General"),
        DescriptionAttribute("ListView")]
        public ListView View
        {
            get { return this.listView; }
            set { this.listView = value; }
        }

        const char sItemSeperator = ';';
        const string sParmSeperator = "|";

        public static CheckedValue[] Parse(string str)
        {
            string[] items = str.Split(sItemSeperator);
            CheckedValue[] values = new CheckedValue[items.Length];
            for (int idx = 0; idx != items.Length; idx++)
            {
                values[idx] = new CheckedValue() { enable = false, value = string.Empty };
                string[] parameters = items[idx].Split(sParmSeperator[0]);
                if (parameters.Length == 2 && bool.TryParse(parameters[0], out values[idx].enable))
                {
                    values[idx].value = parameters[1];
                }
            }

            return values;
        }

        public static String ToString(CheckedValue[] list)
        {
            string result = string.Empty;
            foreach (CheckedValue ckValue in list)
            {
                if (result.Length != 0)
                    result += sItemSeperator;
                result += string.Join(sParmSeperator, new string[] { ckValue.enable.ToString(), ckValue.value });
            }

            return result;
        }

        private void addBtn_Click(object sender, EventArgs e)
        {
            ListViewItem lvItem = this.listView.Items.Add("<new>");
            lvItem.EnsureVisible();
        }

        private void delBtn_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem lvItem in this.listView.SelectedItems)
            {
                this.listView.Items.Remove(lvItem);
            }
        }

        private void listView_KeyUp(object sender, KeyEventArgs e)
        {
            Point pt = this.listView.PointToClient(System.Windows.Forms.Control.MousePosition);
            ListViewHitTestInfo hitInfo = this.listView.HitTest(pt);
            switch (e.KeyCode)
            {
                case Keys.Delete:
                    if (hitInfo.Item != null)
                        this.listView.Items.Remove(hitInfo.Item);
                    break;
                case Keys.Insert:
                    addBtn_Click(null, EventArgs.Empty);
                    break;
            }
        }

        private void ListView_DoubleClick(object sender, EventArgs e)
        {
            Point p = System.Windows.Forms.Control.MousePosition;
            ListView listView = (ListView)sender;
            if (listView != null)
            {
                p = listView.PointToClient(p);
                ListViewHitTestInfo hitInfo = listView.HitTest(p);
                ListViewEdit(listView, hitInfo.Item);
            }
        }

        private bool ListViewEdit(ListView listView, ListViewItem item)
        {
            Control c = listView;

            FieldBox fieldBox = new FieldBox();
            fieldBox.FieldText = item.Text;
            fieldBox.Location = c.PointToScreen(item.Bounds.Location);
            fieldBox.Show();
            fieldBox.Width = item.Bounds.Width;  // reset width
            fieldBox.Visible = false;

            if (fieldBox.ShowDialog() == DialogResult.OK)
            {
                item.Text = fieldBox.FieldText;
                return true;
            }
            return false;
        }

    }

}
