﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace UrlCleaner
{
    /// <summary>
    /// Class found on the web to read and write .ini files which have the same structure as URL files.
    /// 
    /// Author:  Prashant Khandelwal
    /// http://midnightprogrammer.net/post/ReadWrite-settings-to-INI-File-using-C.aspx
    /// 
    /// Used by: Dennis Lang
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// Example of Url files:
    ///  
    ///     [{000214A0-0000-0000-C000-000000000046}]
    ///     Prop3=19,0
    ///     [InternetShortcut]
    ///     URL=steam://rungameid/34330
    ///     IDList=
    ///     IconFile=C:\Program Files (x86)\Steam\steam\games\71a76cd2fbcd1457887fe57727aa10e5e1ce2ea4.ico
    ///     IconIndex=0
    /// 
    /// Another example:
    ///     [InternetShortcut]
    ///     URL=http://technet.microsoft.com/en-us/library/cc723564.aspx
    ///      
    /// </summary>
    class IniFile
    {
        private string filePath;

        [DllImport("kernel32")]
        private static extern long WritePrivateProfileString(string section,
        string key,
        string val,
        string filePath);

        [DllImport("kernel32")]
        private static extern int GetPrivateProfileString(string section,
        string key,
        string def,
        StringBuilder retVal,
        int size,
        string filePath);

        public IniFile(string filePath)
        {
            this.filePath = filePath;
        }

        public void Write(string section, string key, string value)
        {
            WritePrivateProfileString(section, key, value.ToLower(), this.filePath);
        }

        public string Read(string section, string key)
        {
            StringBuilder SB = new StringBuilder(255);
            int i = GetPrivateProfileString(section, key, "", SB, 255, this.filePath);
            return SB.ToString();
        }

        public string FilePath
        {
            get { return this.filePath; }
            set { this.filePath = value; }
        }
    }
}
